package lu.tudor.santec.dicom.gui.header;

import lu.tudor.santec.dicom.gui.header.ge.GEUnifier;
import lu.tudor.santec.dicom.gui.header.philipps.PhilippsUnifier;
import lu.tudor.santec.dicom.gui.header.siemens.SiemensUnifier;

public abstract class DicomHeaderUnifier {
    
    public static String MANUFACTURER = 	"0008,0070";
    public static String MODEL = 			"0008,1090";
    public static String MODALITY = 		"0008,0060";
    
    public static String TABLEFEED_PER_ROTATION = 	"0018,9310";
    public static String PITCH = 					"0018,9311";
    public static String COLLIMATION_TOTAL = 		"0018,9307";
    public static String COLLIMATION_SINGLE = 		"0018,9306";
    
    public static DicomHeaderUnifier siemensUnifier = new SiemensUnifier();
    public static DicomHeaderUnifier philippsUnifier = new PhilippsUnifier();
    public static DicomHeaderUnifier geUnifier = new GEUnifier();
    
    /**
     * returns a unified version of the DicomHeader by applying different unifier classes 
     * specific for the different manufacturers which fill calculate /add/modify some fields
     * 
     * @param header the original DicomHeader
     * @return the unified DicomHeader
     */
    public static DicomHeader getUnifiedHeader(DicomHeader header) {
	DicomHeader dh = header.clone();
	
	DicomHeaderUnifier unifier = null;
	
	// apply the unifier class for the specific manufacturer
	String manufacturer = dh.getHeaderStringValue(MANUFACTURER).toUpperCase();
	if (manufacturer.equals("SIEMENS")) {
	    unifier = siemensUnifier;
	} else if (manufacturer.equals("PHILLIPS")) {
	    unifier = philippsUnifier;
	} else if (manufacturer.equals("GE MEDICAL SYSTEMS")) {
	    unifier = geUnifier;
	}
	
	// if no unifier implemented for this manufacturer -> return the defautl header
	if (unifier == null)
	    return dh;
	
	// unify the general header fields
	unifier.unifyGeneralHeader(dh);
	
	// apply the modality specific modifier
	String modality = dh.getHeaderStringValue(MODALITY).toUpperCase();
	if (modality.equals("CT")) {
	    unifier.unifyCTHeader(dh);
	}
	
	return dh;
    }
    
    /**
     * unifies the general header fields, needs to be implemented in every 
     * manufacturer specific unifier
     * @param dh the dicom header to unify
     */
    protected abstract void unifyGeneralHeader(DicomHeader dh);

    /**
     * unifies the CT header fields, needs to be implemented in every 
     * manufacturer specific unifier
     * @param dh the dicom header to unify
     */
    protected abstract void unifyCTHeader(DicomHeader dh);
    
    
}
