package lu.tudor.santec.dicom.gui.header;

import org.dcm4che2.data.DicomElement;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.util.StringUtils;

public class HeaderTag {
		
		public String tagType = "";
		
		public String tagNr = null;

		public String tagName = null;

		public String tagValue = null;

		private int length;

		private int location;

		private DicomElement dicomElement;

		/**
		 * @return the dicomElement
		 */
		public DicomElement getDicomElement() {
		    return dicomElement;
		}

		/**
		 * @param dicomElement the dicomElement to set
		 */
		public void setDicomElement(DicomElement dicomElement) {
		    this.dicomElement = dicomElement;
		}

		public HeaderTag() {
			
		};
		
		public HeaderTag(String headerLine) {
			if (headerLine.length() > 9)
				try {
					int idx1 = headerLine.indexOf(" ");
					int idx2 = headerLine.indexOf(": ", idx1);
					tagNr = headerLine.substring(0, idx1);
					tagName = headerLine.substring(idx1 + 2, idx2).trim();
					tagValue = headerLine.substring(idx2 + 2, headerLine.length()).trim();
				} catch (Exception e) {
//					e.printStackTrace();
					tagValue = headerLine;
				}
				else
					tagValue = headerLine;
		}
		
		public HeaderTag(String tagNr, String tagName, String tagValue) {
				this.tagNr = tagNr;
				this.tagName = tagName;
				if (tagName == null || tagName.equals("")) {
					this.tagName = DicomHeader.getHeaderName(tagNr);
				}
				this.tagValue = tagValue;
		}
		
		public HeaderTag(String tagNr, String tagName, String tagValue, String tagType) {
			this.tagNr = tagNr;
			this.tagName = tagName;
			if (tagName == null || tagName.equals("")) {
				this.tagName = DicomHeader.getHeaderName(tagNr);
			}
			this.tagValue = tagValue;
			this.tagType = tagType;
		}
		
		public HeaderTag(String tagNr, String tagType, String tagName, String tagValue, int location, int length) {
			this.tagType = tagType;
			this.tagNr = tagNr;
			this.tagName = tagName;
			if (tagName == null || tagName.equals("")) {
				this.tagName = DicomHeader.getHeaderName(tagNr);
			}
			if (tagValue != null)
				this.tagValue = tagValue.replace((char)0, ' ');
			this.location = location;
			this.length = length;
	}
		
		public HeaderTag(DicomElement element, DicomObject dObj) {
		    this.dicomElement = element;
		    int tag = element.tag();
			this.tagNr = StringUtils.shortToHex(tag >> 16) + 
				',' + StringUtils.shortToHex(tag);
			this.tagType = element.vr().toString();
			try {
			    tagValue = element.getString(dObj.getSpecificCharacterSet(),false);		
			    if (tagValue == null)
				tagValue = "";
			} catch (Exception e) {
			}
			tagName = DicomHeader.getHeaderName(tagNr);
		}

		public String toString() {
			return tagNr + " " + (tagType!=""?"["+tagType+"] ":"") +tagName + ": " + tagValue;
		}

		/**
		 * @return the length
		 */
		public int getLength() {
			return length;
		}

		/**
		 * @return the location
		 */
		public int getLocation() {
			return location;
		}

		/**
		 * @return the tagName
		 */
		public String getTagName() {
			return tagName;
		}

		/**
		 * @param tagName the tagName to set
		 */
		public void setTagName(String tagName) {
			this.tagName = tagName;
		}

		/**
		 * @return the tagNr
		 */
		public String getTagNr() {
			return tagNr;
		}

		/**
		 * @param tagNr the tagNr to set
		 */
		public void setTagNr(String tagNr) {
			this.tagNr = tagNr;
		}

		/**
		 * @return the tagType
		 */
		public String getTagType() {
			return tagType;
		}

		/**
		 * @param tagType the tagType to set
		 */
		public void setTagType(String tagType) {
			this.tagType = tagType;
		}

		/**
		 * @return the tagValue
		 */
		public String getTagValue() {
			return tagValue;
		}

		/**
		 * @param tagValue the tagValue to set
		 */
		public void setTagValue(String tagValue) {
			this.tagValue = tagValue;
		}
}