package lu.tudor.santec.dicom.gui.selector;

import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.Vector;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.ProgressMonitor;

import lu.tudor.santec.dicom.gui.header.DicomHeader;
import lu.tudor.santec.dicom.gui.header.HeaderTag;
import lu.tudor.santec.i18n.Translatrix;

public class DicomMatcher {

	/**
	 * static logger for this class
	 */
	private static Logger logger = Logger.getLogger(DicomMatcher.class
			.getName());
	
	private HeaderTag[] headerTags;
	private Component parent;
	private ProgressMonitor progressMonitor;

	public DicomMatcher(Component parent)  {
		this.parent = parent;
	}
	
	public void setHeaderTags(HeaderTag[] tags) {
		this.headerTags = tags;
	}
	
	public Vector findMachingFiles(File dir) {
		ArrayList al = new ArrayList();
		visitAllFiles(dir, al);
		Vector matches = new Vector();
		
		if (parent != null)
			progressMonitor = new ProgressMonitor(parent, Translatrix.getTranslationString("dicom.searchingFiles"), "", 0, al.size());
		
		for (int i = 0; i < al.size(); i++) {
			if (parent != null && progressMonitor.isCanceled()) {
				break;
			}
			try {
				DicomFile dFile = new DicomFile((File)al.get(i));
				if (parent != null) {
					progressMonitor.setNote("img " + dFile.getFile().getName()
						+ " ( " + (i + 1) + " of " + (al.size() + 1)
						+ " )");
					progressMonitor.setProgress(i);
				}
				if (matchFile(dFile))
					matches.add(dFile);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (parent != null)
			progressMonitor.close();
		return matches;
	}
		
	
	
	public boolean matchFile(DicomFile dfile) {
		try {
			DicomHeader dh = dfile.getDicomHeader();
			for (int i = 0; i < headerTags.length; i++) {
				if (! dh.containsHeaderTag(headerTags[i].getTagNr())) {
					logger.fine("missing tag: " + headerTags[i].getTagNr());
					return false;
				}
				String isVal = dh.getHeaderStringValue(headerTags[i].getTagNr()).trim();
				String shouldExpr = headerTags[i].getTagValue(); 
				logger.fine(isVal + " : " + shouldExpr);
				Pattern p = Pattern.compile(shouldExpr, Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(isVal);
				if (m.find()) {
					logger.fine("  found!");
				} else {
					logger.fine("");
					return false;
				}
			}
			return true;			
		} catch (Exception e) {
			logger.info("no DICOM file: " + dfile.getFile().getName());
			return false;
		}
	}

	//	 Process only files under dir
	private static void visitAllFiles(File dir, ArrayList al) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                visitAllFiles(new File(dir, children[i]), al);
            }
        } else {
            al.add(dir);
        }
    }
	
}
